## Distributed as part of the supplementary material for the manuscript
## "Jeffreys-prior penalty, finiteness and shrinkage in binomial-response generalized linear models"
##
## Authors: Ioannis Kosmidis, David Firth
## Date: 19 March 2020
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("ggplot2")
source("jeffreys-MPL.R")

omega <- function(pi, link = "logit") {
    f <- binomial(link)
    eta <- f$linkfun(pi)
    f$mu.eta(eta)^2/(pi*(1 - pi))
}

LogDetJef <- function(pi1, pi2, link = "logit") {
    f <- binomial(link)
    pis <- c(pi1, pi2)
    etas <- f$linkfun(pis)
    w <- m*f$mu.eta(etas)^2/(pis*(1 - pis))
    F <- t(w*X)%*%X
    log(det(F))
}

wfun <- function(pis, link = "logit") {
    f <- binomial(link)
    etas <- f$linkfun(pis)
    f$mu.eta(etas)^2/(pis*(1 - pis))
}

m <- c(9, 9)
x <- c(-1, 1)
X <- cbind(1, x)

links <- c("logit", "probit", "cloglog", "cauchit")

## Contours
probs <- seq(1e-03, 1 - 1e-03, length = 100)
contours_df <- expand.grid(p1 = probs, p2 = probs, link = links, stringsAsFactors = FALSE)
contours_df$density <- with(contours_df, Vectorize(LogDetJef)(p1, p2, link))
contours_df$link <- factor(contours_df$link, levels = links)

## Arrows
fitted_df <- expand.grid(y1 = 0:m[1], y2 = 0:m[2], link = links, method = c("ML", "RB"),
                         stringsAsFactors = FALSE)
fitted_df$p1_ml  <- fitted_df$p2_ml <- fitted_df$p1_br  <- fitted_df$p2_br <- NA
for (j in seq.int(nrow(fitted_df))) {
    dat <- with(fitted_df[j, ], data.frame(y = c(y1, y2), x = x, m = m))
    clink <- fitted_df[j, "link"]
    m_ml <- glm(y/m ~ x, weights = m, family = binomial(clink), data = dat,
                etastart = pcauchy(y/m))
    fits <- fitted(m_ml)
    fitted_df[j, "p1_ml"] <- fits[1]
    fitted_df[j, "p2_ml"] <- fits[2]
    res <- JeffreysMPL(y = dat$y, m = dat$m, X = X, adj = 0.01, a = 0.5, epsilon = 1e-10,
                       link = clink)
    fits <- res$fitted.values
    fitted_df[j, "p1_br"] <- fits[1]
    fitted_df[j, "p2_br"] <- fits[2]
}
fitted_df$link <- factor(fitted_df$link, levels = links)


## Figure 2; right
## maximum omega
max_df <- data.frame(link = links, stringsAsFactors = FALSE)
max_df$z1 <- max_df$z2 <- sapply(max_df$link, function(clink) {
    optimize(omega, c(0, 1), maximum = TRUE, link = clink)$maximum
})
max_df$link <- factor(max_df$link, levels = links)

g3 <- ggplot(contours_df) +
    geom_contour(aes(x = p1, y = p2, z = density), bins = 100, col = "gray") +
    facet_wrap(~ link, ncol = 2) +
    geom_segment(data = fitted_df, aes(x = p1_ml, xend = p1_br, y = p2_ml, yend = p2_br),
                 arrow = arrow(length = unit(0.1, "cm")), size = 0.2) +
    geom_point(data = max_df, aes(x = z1, y = z2)) +
    theme_minimal() +
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    labs(x = expression(pi[1]), y = expression(pi[2]))

## Figure 2; left
probs <- seq(0, 1, length = 10000)
weights_df <- NULL
for (clink in links) {
    max <- optimize(wfun, c(0, 1), link = clink, maximum = TRUE)$max
    ws <- sapply(probs, function(y) wfun(y, clink))
    weights_df <- rbind(weights_df,
                        data.frame(p = probs, weights = ws, link = clink))
}
weights_df$link <- factor(weights_df$link,
                          levels = c("logit", "probit", "cloglog", "cauchit"))

g4 <- ggplot(weights_df) +
    geom_line(aes(x = p, y = weights)) +
    geom_vline(data = max_df, aes(xintercept = z1), lty = 2) +
    geom_text(data = max_df, aes(x = 0.1, y = 0.9,
                                 label = paste("z[0] ==", round(z1, 3))),
              parse = TRUE, hjust = "left") +
    facet_wrap(~ link, ncol = 2) +
    lims(x = c(1e-05, 1 - 1e-05), y = c(0, 1)) +
    theme_minimal() +
    labs(x = "z", y = expression(bar(omega)(z)))

print(g3)
print(g4)

## library("gridExtra")
## pdf("shrinkage.pdf", width = 10, height = 5)
## grid.arrange(g4, g3, nrow = 1, widths = c(2, 2))
## dev.off()
